/*
 Return a list of nodes in a order of breath search. (The graph must be cycle free.) 
*/
List<T> breathSearch<T>(T root, Iterable<T> getChildren(T node)) {
  if (root == null) throw new ArgumentError.notNull("root");
  if (getChildren == null) throw new ArgumentError.notNull("getChildren");

  var result = List.of([root]);
  var children = getChildren(root);

  if (children == null || children.length == 0) return result;

  var iterator = children.iterator;

  while (true) {
    var allChildren = Set<T>();
    while (iterator != null && iterator.moveNext()) {
      var current = iterator.current;

      if (result.contains(current)) continue;

      result.add(current);

      children = getChildren(current);
      if (children == null) continue;

      allChildren.addAll(children);
    }

    if (allChildren.length == 0) break;

    iterator = allChildren.iterator;
  }

  return result;
}
