/*
 Return a list of nodes in a order of pre depth search. (The graph must be cycle free.) 
*/
List<T> preDepthSearch<T>(T root, Iterable<T> getChildren(T node)) {
  if (root == null) throw new ArgumentError.notNull("root");
  if (getChildren == null) throw new ArgumentError.notNull("getChildren");

  var result = List.of([root]);
  var children = getChildren(root);

  if (children == null || children.length == 0) return result;

  var iterators = new List<Iterator<T>>();
  
  iterators.add(children.iterator);

  while (iterators.length > 0) {
    var iterator = iterators.last;
    while (iterator != null && iterator.moveNext()) {
      var current = iterator.current;

      if(result.contains(current))
        continue;

      result.add(current);
      
      children = getChildren(current);
      if(children == null)
        continue;

      iterator = children.iterator;
      iterators.add(iterator);
    }
    iterators.remove(iterator);
  }

  return result;
}

/*
 Return a list of nodes in a order of post depth search. (The graph must be cycle free.) 
*/
List<T> postDepthSearch2<T>(T root, Iterable<T> getChildren(T node)) {
  if (root == null) throw new ArgumentError.notNull("root");
  if (getChildren == null) throw new ArgumentError.notNull("getChildren");

    var children = getChildren(root);

  if (children == null || children.length == 0) return List.of([root]);

  var result = <T>[];
  var iterators = new List<Iterator<T>>();
  
  iterators.add(children.iterator);

  while (iterators.length > 0) {
    var iterator = iterators.last;
    var prevValue = iterator.current;

    while (iterator != null && iterator.moveNext()) {
      var current = iterator.current;

      if(result.contains(current))
        continue;

      children = getChildren(current);
      if(children == null)
        continue;

      iterator = children.iterator;
      iterators.add(iterator);
    }

    if(prevValue != null && !result.contains(prevValue))
      result.add(prevValue);

    iterators.remove(iterator);
  }

  result.add(root);
  return result;
}
